from random import choice
import re
from . import Arr


def __case(variable: str = '', operator: str = '') -> str:
    if ' ' in variable:
        variables = variable.split(' ')
        variable = ''.join(v.capitalize() for v in variables)

    return re.sub(r'(?<!^)(?=[A-Z])', operator, variable).lower()


def kebabcase(variable: str = '') -> str:
    return __case(variable, '-')


def camelcase(variable: str = '') -> str:
    word = pascalcase(variable)
    return word[0].lower() + word[1:]


def pascalcase(variable: str = '') -> str:
    regex = re.compile("[^A-Za-z]+")
    variable = regex.split(variable)
    wordsTitle = list(map(lambda x: x.title(), variable))
    return ''.join(wordsTitle)


def snakecase(variable: str = '') -> str:
    return __case(variable, '_')


def isStr(variable=None) -> bool:
    return isinstance(variable, str)


def isEmpty(variable: str = None) -> bool:
    if type(variable) == int:
        return False

    return False if variable is not None and len(variable) > 0 else True


def isAscii(variable=None) -> bool:
    try:
        variable.decode('ascii')
    except:
        return False
    else:
        return True


def limit(variable: str = '', length: int = None, placeholder='') -> str:
    length = length if length is not None else len(variable)
    return (variable[:length] + placeholder) if len(variable) > length else variable


def words(variable: str = '', length: int = None, placeholder='') -> str:
    word = variable.split(' ')
    length = length if length is not None else len(word)
    newWord = ' '.join(word[0:length])
    return newWord + placeholder


def random(length: int = 16) -> str:
    numberLetters = '0123456789'
    stringLetters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
    ln = 3 if length >= 3 else 1

    random_string = ''.join(list(map(lambda x: choice(stringLetters), range(ln))))
    random_string += ''.join(list(map(lambda x: choice(stringLetters + numberLetters), range(length - ln))))

    return random_string


def start(variable: str = '', check: str = None) -> str:
    if len(variable) > 0:
        variable = check + variable if variable[0] != check else variable

    return variable


class of:

    def __init__(self, variable: str = ''):
        self.__variable = variable

    def append(self, variable: str = ''):
        self.__variable += variable
        return self

    def prepend(self, variable: str = ''):
        self.__variable = variable + self.__variable
        return self

    def replace(self, v1: str = None, v2: str = None):
        self.__variable = self.__variable.replace(v1, v2)
        return self

    def replaceFirst(self, v1: str = None, v2: str = None):
        self.__variable = self.__variable.replace(v1, v2, 1)
        return self

    def __reversed(self):
        self.__variable = self.__variable[len(self.__variable)::-1]

    def replaceLast(self, v1: str = None, v2: str = None):
        self.__reversed()
        self.replaceFirst(v1, v2)
        self.__reversed()
        return self

    def replaceList(self, param: str = None, replaces: list = []):
        for x in replaces:
            self.replaceFirst(param, x)
        return self

    def replaceMatches(self, match: str = None, variable: str = None):
        regex = re.compile(match)
        variables = regex.split(self.__variable)
        variables = Arr.array_filter(variables)
        self.__variable = variable.join(variables)
        return self

    def after(self, param: str = None):
        if isEmpty(param):
            return self

        first = self.__variable.find(param)
        self.__variable = self.__variable[first+len(param):] if first != -1 else self.__variable
        return self

    def afterLast(self, param: str = None):
        if isEmpty(param):
            return self

        first = self.__variable.rfind(param)
        self.__variable = self.__variable[first+len(param):] if first != -1 else self.__variable
        return self

    def before(self, param: str = None):
        if isEmpty(param):
            return self

        first = self.__variable.find(param)
        self.__variable = self.__variable[:first] if first != -1 else self.__variable
        return self

    def beforeLast(self, param: str = None):
        if isEmpty(param):
            return self

        first = self.__variable.rfind(param)
        self.__variable = self.__variable[:first] if first != -1 else self.__variable
        return self

    def getValue(self) -> str:
        return self.__variable



from random import choice
from . import Str

def __throw(exception):
    raise BaseException(exception)


def get(arr: list or dict = None, key_path: str = '') -> list or None:
    keys = key_path.split('.')

    def dictGet(arr):
        for k in filter(None, keys):
            if k in arr:
                arr = arr[k]
            else:
                return None
        return arr

    def listGet(arr):
        arrs = []
        for item in arr:
            if type(item) == dict:
                add = dictGet(item)
                arrs.append(add)
        return arrs

    if type(arr) == dict:
        return dictGet(arr)
    elif type(arr) == list:
        return listGet(arr)


def column(arr: list = [], key: any = None) -> list or None:
    _raise = 'no {key} in list'.format(key=key)

    if '.' in key:
        return get(arr, key)

    firstType = type(first(arr))

    if firstType in [tuple, list]:
        return list(map(lambda x: x[key] if len(x) - 1 >= key else __throw(_raise), arr))
    elif firstType in [dict]:
        return list(map(lambda x: x[key] if key in x else __throw(_raise), arr))
    else:
        return None


def first(arr: list = []):
    return arr[0]


def last(arr: list = []):
    return arr[-1]


def combine(keys: list = [], values: list = []) -> dict:
    if len(keys) != len(values):
        raise BaseException('keys and values are not equal in number')

    return {x: values[k] for k, x in enumerate(keys)}


def divide(arr: dict = {}) -> tuple:
    return list(arr.keys()), list(arr.values())


def has(arr: dict = {}, key_path: str = '') -> bool:
    check = get(arr=arr, key_path=key_path)

    if type(check) == list:
        check = array_filter(check)
        if len(check) == 0:
            return False

    if check is not None:
        return True
    else:
        return False


def only(arr: dict = {}, keys: list = []) -> dict:
    return {x: arr[x] if x in list(arr.keys()) else __throw('no {key} in list'.format(key=x)) for x in keys}


def pluck(arr: list = [], keys: str = None, vals: str = None) -> dict or list:
    def one():
        allKeys = column(arr=arr, key=keys)
        allValues = column(arr=arr, key=vals)
        return combine(allKeys, allValues)

    def multiple(arr, keys, vals):
        keys = get(arr, keys)
        vals = get(arr, vals)
        return combine(keys, vals)

    if keys is not None and vals is not None:
        if '.' not in keys and '.' not in vals:
            return one()

        elif '.' in keys and '.' in vals:
            return multiple(arr, keys, vals)

    elif keys is not None and vals is None:
        if '.' not in keys:
            return column(arr, keys)

        else:
            keysGet = get(arr, keys)
            valAll = [None for i in range(0, len(keysGet))]
            return combine(keysGet, valAll)

    elif keys is None and vals is not None:
        if '.' not in vals:
            return column(arr, vals)

        else:
            return get(arr, vals)


def prepend(arr: list or dict = [], val: any = None) -> list or dict:
    arr.insert(0, val)
    return arr


def exists(arr: dict or list = None, search: any = None, key: str = None) -> bool:
    srch = arr
    if key is not None:
        srch = get(arr, key)
    elif type(arr) == dict:
        srch = arr.values()

    if search in srch:
        return True
    else:
        return False


def keyExists(arr: dict = {}, val: any = None) -> bool:
    if '.' in val and get(arr, val) is not None:
        return True
    elif val in list(arr.keys()):
        return True
    else:
        return False


def random(arr: list = None):
    return choice(arr)


def wrap(key: str or None = None, val: any = None) -> list or dict:
    if key is None:
        return list()

    elif '.' in key:
        arr = key.split('.')
        news = None
        for s, it in enumerate(reversed(arr)):
            if s == 0:
                news = {it: val}
            else:
                news = dict({it: news})

        return news

    else:
        return [key]


def array_filter(arr: list = []) -> list:
    return list(filter(lambda x: x is not None and x != '', arr))




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fthelper",
    version="0.1.2",
    author="Fatih IRDAY",
    author_email="fatihirday@gmail.com",
    description="Dict, list and string helper package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/fatihirday/fthelper",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
